@@ -1,5 +1,26 @@
 
-/* Author : Stephen Smalley, <sds@epoch.ncsc.mil> */
+/* Author : Stephen Smalley, <sds@tycho.nsa.gov> */
+
+/*
+ * Updated : Karl MacMillan <kmacmillan@mentalrootkit.com>
+ *
+ * Copyright (C) 2007 Red Hat, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 
 /* FLASK */
 
@@ -12,10 +33,10 @@
 #include <sepol/policydb/hashtab.h>
 
 hashtab_t hashtab_create(unsigned int (*hash_value) (hashtab_t h,
-						     const hashtab_key_t key),
+						     const_hashtab_key_t key),
 			 int (*keycmp) (hashtab_t h,
-					const hashtab_key_t key1,
-					const hashtab_key_t key2),
+					const_hashtab_key_t key1,
+					const_hashtab_key_t key2),
 			 unsigned int size)
 {
 
@@ -48,7 +69,7 @@ int hashtab_insert(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum)
 	hashtab_ptr_t prev, cur, newnode;
 
 	if (!h)
-		return HASHTAB_OVERFLOW;
+		return SEPOL_ENOMEM;
 
 	hvalue = h->hash_value(h, key);
 	prev = NULL;
@@ -59,11 +80,11 @@ int hashtab_insert(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum)
 	}
 
 	if (cur && (h->keycmp(h, key, cur->key) == 0))
-		return HASHTAB_PRESENT;
+		return SEPOL_EEXIST;
 
 	newnode = (hashtab_ptr_t) malloc(sizeof(hashtab_node_t));
 	if (newnode == NULL)
-		return HASHTAB_OVERFLOW;
+		return SEPOL_ENOMEM;
 	memset(newnode, 0, sizeof(struct hashtab_node));
 	newnode->key = key;
 	newnode->datum = datum;
@@ -76,7 +97,7 @@ int hashtab_insert(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum)
 	}
 
 	h->nel++;
-	return HASHTAB_SUCCESS;
+	return SEPOL_OK;
 }
 
 int hashtab_remove(hashtab_t h, hashtab_key_t key,
@@ -87,7 +108,7 @@ int hashtab_remove(hashtab_t h, hashtab_key_t key,
 	hashtab_ptr_t cur, last;
 
 	if (!h)
-		return HASHTAB_MISSING;
+		return SEPOL_ENOENT;
 
 	hvalue = h->hash_value(h, key);
 	last = NULL;
@@ -98,7 +119,7 @@ int hashtab_remove(hashtab_t h, hashtab_key_t key,
 	}
 
 	if (cur == NULL || (h->keycmp(h, key, cur->key) != 0))
-		return HASHTAB_MISSING;
+		return SEPOL_ENOENT;
 
 	if (last == NULL)
 		h->htable[hvalue] = cur->next;
@@ -109,7 +130,7 @@ int hashtab_remove(hashtab_t h, hashtab_key_t key,
 		destroy(cur->key, cur->datum, args);
 	free(cur);
 	h->nel--;
-	return HASHTAB_SUCCESS;
+	return SEPOL_OK;
 }
 
 int hashtab_replace(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum,
@@ -120,7 +141,7 @@ int hashtab_replace(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum,
 	hashtab_ptr_t prev, cur, newnode;
 
 	if (!h)
-		return HASHTAB_OVERFLOW;
+		return SEPOL_ENOMEM;
 
 	hvalue = h->hash_value(h, key);
 	prev = NULL;
@@ -138,7 +159,7 @@ int hashtab_replace(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum,
 	} else {
 		newnode = (hashtab_ptr_t) malloc(sizeof(hashtab_node_t));
 		if (newnode == NULL)
-			return HASHTAB_OVERFLOW;
+			return SEPOL_ENOMEM;
 		memset(newnode, 0, sizeof(struct hashtab_node));
 		newnode->key = key;
 		newnode->datum = datum;
@@ -151,10 +172,10 @@ int hashtab_replace(hashtab_t h, hashtab_key_t key, hashtab_datum_t datum,
 		}
 	}
 
-	return HASHTAB_SUCCESS;
+	return SEPOL_OK;
 }
 
-hashtab_datum_t hashtab_search(hashtab_t h, const hashtab_key_t key)
+hashtab_datum_t hashtab_search(hashtab_t h, const_hashtab_key_t key)
 {
 
 	int hvalue;
@@ -206,7 +227,7 @@ int hashtab_map(hashtab_t h,
 	hashtab_ptr_t cur;
 
 	if (!h)
-		return HASHTAB_SUCCESS;
+		return SEPOL_OK;
 
 	for (i = 0; i < h->size; i++) {
 		cur = h->htable[i];
@@ -217,7 +238,7 @@ int hashtab_map(hashtab_t h,
 			cur = cur->next;
 		}
 	}
-	return HASHTAB_SUCCESS;
+	return SEPOL_OK;
 }
 
 void hashtab_map_remove_on_error(hashtab_t h,
